@@ -17,6 +17,7 @@ use crate::{
     index::SparseIndexInformation,
     utils::buffer::{Buffer, MemoryBuffer, MmapBuffer, Slice},
 };
+use indicatif::{ProgressIterator, ProgressStyle};
 use log::{debug, info};
 use serde::{Deserialize, Serialize};
 
@@ -451,9 +452,14 @@ impl SparseIndex for CompressedIndex {
     ) -> Box<dyn super::index::BlockTermImpactIterator + '_> {
         Box::new(CompressedBlockTermImpactIterator::new(self, term_index))
     }
-    
+
     fn max_doc_id(&self) -> DocId {
-        self.information.terms.iter().map(|term| term.max_doc_id).max().unwrap_or(0)
+        self.information
+            .terms
+            .iter()
+            .map(|term| term.max_doc_id)
+            .max()
+            .unwrap_or(0)
     }
 }
 
@@ -476,7 +482,7 @@ pub struct CompressionTransform {
     #[doc = r"Document ID compressor"]
     pub doc_ids_compressor_factory: Box<dyn DocIdCompressorFactory>,
 
-    #[doc = r"maximum number of records per block"]
+    #[doc = r"Impact value compressor"]
     pub impacts_compressor_factory: Box<dyn ImpactCompressorFactory>,
 }
 
@@ -521,7 +527,7 @@ impl IndexTransform for CompressionTransform {
         let mut docid_position = 0;
 
         // Iterate over terms
-        for term_index in 0..index.len() {
+        for term_index in (0..index.len()).progress() {
             // Read everything
             let mut it = index.iterator(term_index);
             let mut flag = true;
